#!/bin/sh
## build glibc_$VERSION for KISS Linux
## Gilbert Ashley <amigo@ibiblio.org>
# Script Version = 06

# This script builds glibc and splits it into several main packages:
# glibc		-the runtime libraries
# glibc-devel	- the headers and other files needed for development
# glibc-zoneinfo	- needed for setting the timezone of your system
# glibc-i18n	- the basic structure for localization using languages besides english
# glibc-locale	-individual packages for each locale

CWD=$(pwd)

VERSION=2.11.1
# libidn now included with glibc>=2.11
# LIBIDN_VERSION=2.9
# LIBIDN_SOURCE=glibc-libidn-$LIBIDN_VERSION.tar.xz
CVS_VERSION=2.11.1
# edit and uncomment to use the glibc-ports
# PORTS_VERSION=2.11.1

# be sure the kernel headers are installed in the right place:
# --with-headers=/usr/src/linux-${KERNEL_HEADERS}/usr/include
KERNEL_HEADERS=2.6.31.11
MIN_KERNEL=2.6.18
TZDATA=tzdata2009u
TZCODE=tzcode2009t
BUILD=${BUILD:-3}
# BUILD 2 -add KERNEL_HEADERS, MIN_KERNEL, TZDATA and TZCODE info to glibc meta-data
# add dummy code for handling ports sources and meta-data

PACKAGER="Gilbert Ashley <amigo@ibiblio.org>"

ARCH=${ARCH:-i586}

TARGET_OS=$ARCH-kiss-linux
PKG_META=pkg-meta
PKG_FORMAT=tpkg
MAN_COMPRESS='gzip -9'

# PATCHLIST
# Use old-style locale directories rather than a single /usr/lib/locale/locale-archive file:
  ## glibc.locale.no-archive.diff.gz
# The is_IS locale is causing a strange error about the "echn" command
# not existing.  This patch reverts is_IS to the version shipped in glibc-2.5:
  ## is_IS.diff.gz
# Fix NIS netgroups:
  ## glibc.nis-netgroups.diff.gz
# This patch makes glibc consider only the first three digits of the
# kernel version and drops any trailing non-digit characters:
  ## glibc.kernelversion.diff.gz | patch -p1 --verbose || exit 1
# Support ru_RU.CP1251 locale:
  ## glibc.ru_RU.CP1251.diff.gz | patch -p1 --verbose || exit 1
# Fix missing MAX macro in getcwd.c:
  ## glibc.getcwd.max.macro.diff.gz | patch -p1 --verbose || exit 1
# Fix a dumb warning ( from the crosstool-ng glibc fixes)
  # i386-preferred-stack-boundary.patch.gz

PATCHLIST='glibc.locale.no-archive.diff.gz
glibc.nis-netgroups.diff.gz
glibc.ru_RU.CP1251.diff.gz
glibc.getcwd.max.macro.diff.gz
i386-preferred-stack-boundary.patch.gz
glibc-2.11.1-etc-resolv.conf.diff.gz'

# Temporary build location.
TMP=${TMP:-$CWD/Glibc-Project}
if [ ! -d $TMP ]; then
  mkdir -p $TMP
fi

# The kernel-headers version should be carefully selected to go with the glibc version
if [ ! -d /usr/src/linux-${KERNEL_HEADERS}/usr/include ]; then
  echo "Missing:  /usr/src/linux-${KERNEL_HEADERS}"
  echo "Exiting..."
  exit
fi

# set the compiler optimization and LIBDIRSUFFIX
LIBDIRSUFFIX=
case $ARCH in
	i386)	OPTIMIZE="-O3 -pipe -march=i386 -mcpu=i686" ;;
	i486)	OPTIMIZE="-O3 -pipe -march=i486 -mtune=i686" ;;
	i586)	OPTIMIZE="-O3 -pipe -march=i586 -mtune=i686" ;;
	i686)	OPTIMIZE="-O3 -pipe -march=i686" ;;
	athlon)	OPTIMIZE="-O3 -pipe -march=athlon" ;;
	s390)	OPTIMIZE="-O3 -pipe" ;;
	s390x)	OPTIMIZE="-O3 -pipe" ;;
	x86_64)	OPTIMIZE="-O3 -fPIC -pipe" ; LIBDIRSUFFIX="64" ;;
	*)		OPTIMIZE="-O3" ;;
esac

# This is going to be the initial 'install_root' location
export PKG_TREE=$TMP/package-glibc-incoming-tree
# package-tree for each of the main packages
PKG_MAIN=$TMP/package-glibc
PKG_TIME=$TMP/package-glibc-zoneinfo
PKG_I18N=$TMP/package-glibc-i18n
PKG_DEVEL=$TMP/package-glibc-devel

# create the needed directories
create_dirs() {
# Empty these locations first:
for dir in $PKG_MAIN $PKG_TIME $PKG_I18N $PKG_DEVEL ; do
  if [ -d $dir ]; then
    rm -rf $dir
  fi
  mkdir -p $dir
done

# Create the PKG_TREE (used as install_root)
mkdir -p $PKG_TREE/lib${LIBDIRSUFFIX}
mkdir -p $PKG_TREE/sbin
mkdir -p $PKG_TREE/usr/bin
mkdir -p $PKG_TREE/usr/lib${LIBDIRSUFFIX}
mkdir -p $PKG_TREE/usr/sbin
mkdir -p $PKG_TREE/usr/include
mkdir -p $PKG_TREE/usr/share
mkdir -p $PKG_TREE/usr/share/doc
mkdir -p $PKG_TREE/usr/share/info
mkdir -p $PKG_TREE/var/db/nscd
mkdir -p $PKG_TREE/var/run/nscd
}

# apply pathces from PATCHLIST
apply_patches() {
	cd $TMP/glibc-$CVS_VERSION
	for p in $PATCHLIST ; do
		zcat $CWD/glibc-patches/$p | patch -p1 || exit 1
	done
	# Update the timezone information.
	( cd timezone
		tar xzf $CWD/timezone/$TZDATA.tar.gz
		chown root:root *
		mv yearistype.sh yearistype
		chmod 644 *
		chmod 755 yearistype
		mkdir tzcode
		cd tzcode
		tar xzf $CWD/timezone/$TZCODE.tar.gz
		chown -R root:root .
		chmod 644 *
		cp -a *.c *.h ..
	)
}

# write package meta-data
write_meta_data() {
	THIS_PACKAGE="$1"
	mkdir -p "$THIS_PACKAGE"/install
	case $THIS_PACKAGE in
		$PKG_MAIN) echo "# Package: glibc_$VERSION-$ARCH-$BUILD.$PKG_FORMAT" > "$THIS_PACKAGE"/install/$PKG_META ;;
		$PKG_DEVEL) echo "# Package: glibc-devel_$VERSION-$ARCH-$BUILD.$PKG_FORMAT" > "$THIS_PACKAGE"/install/$PKG_META ;;
	esac
	echo "# Created by: $PACKAGER" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Hostname: $(hostname)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Date: $(date)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Processor: $(uname -i) $(uname -m)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Kernel: $(uname -r) $(uname -v)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Compiler: GCC-$(gcc -dumpversion) $(gcc -dumpmachine)" >> "$THIS_PACKAGE"/install/$PKG_META
	binutils_version=$(ld --version |head -n 1)
	echo "# Binutils:  ${binutils_version##* }" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Kernel/glibc headers:  ${KERNEL_HEADERS}" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Minimum kernel version:  ${MIN_KERNEL}" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Source name: glibc-$CVS_VERSION.tar.xz" >> "$THIS_PACKAGE"/install/$PKG_META
	if [[ "$PORTS_VERSION" != "" ]] ; then
		echo "# glibc ports version: glibc-ports-$PORTS_VERSION.tar.xz" >> "$THIS_PACKAGE"/install/$PKG_META
	fi
	if [[ "$PATCHLIST" != "" ]] ; then
		echo "# Patches applied:" >> "$THIS_PACKAGE"/install/$PKG_META
		for PATCH in $PATCHLIST ; do
			echo "	${PATCH##*/}" >> "$THIS_PACKAGE"/install/$PKG_META
		done
	fi
	echo "# tzdata version: $TZDATA" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# tzcode version $TZCODE:" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Build configuration:" >> "$THIS_PACKAGE"/install/$PKG_META
	cat "$CWD"/config-line.txt >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
}

# Correct library paths in doinst.sh for x86_64.
install_doinst() {
	sed -e "s#SFX#$LIBDIRSUFFIX#g" $CWD/doinst.sh.in > install/doinst.sh
	sed -i "s#VERSION#$VERSION#g" install/doinst.sh
	if [ "$ARCH" = "x86_64" ] ; then
		sed -i "s#ld-linux.so.2#ld-linux-x86-64.so.2#g" install/doinst.sh
	fi
}

# substitute PACKAGER in description files
install_desc() {
	DESC_FILE=$1
	sed -e "s#PACKAGER#$PACKAGER#g" $DESC_FILE > install/pkg-desc
}

# unpack the sources
unpack_sources() {
	cd $TMP
	rm -rf glibc-$CVS_VERSION
	tar xvf $CWD/glibc-$CVS_VERSION.tar.*z*
	# libidn now included with glibc>=2.11
	#cd glibc-$CVS_VERSION
	#tar xf $CWD/$LIBIDN_SOURCE
	#mv glibc-libidn-$LIBIDN_VERSION libidn || exit 1
	if [[ "$PORTS_VERSION" != "" ]] ; then
		( cd glibc-$CVS_VERSION
			tar xvf $CWD/glibc-ports-$PORTS_VERSION.tar.*z*
		)
	fi
}

# correct perms and ownership
fix_source_perms() {
	cd $TMP/glibc-$CVS_VERSION
	chown -R root:root .
	find . -perm 666 -exec chmod 644 {} \;
	find . -perm 664 -exec chmod 644 {} \;
	find . -perm 600 -exec chmod 644 {} \;
	find . -perm 444 -exec chmod 644 {} \;
	find . -perm 400 -exec chmod 644 {} \;
	find . -perm 440 -exec chmod 644 {} \;
	find . -perm 777 -exec chmod 755 {} \;
	find . -perm 775 -exec chmod 755 {} \;
	find . -perm 511 -exec chmod 755 {} \;
	find . -perm 711 -exec chmod 755 {} \;
	find . -perm 555 -exec chmod 755 {} \;
	# Clean up any CVS cruft
	find . -type d -name CVS -exec rm -r {} \; 2> /dev/null
}

# configure and compile the sources
build_sources() {
	cd $TMP/glibc-$CVS_VERSION
	# Make build directory:
	mkdir build-glibc_$VERSION
	cd build-glibc_$VERSION
	
	# added from DIY
	#cp -v ../iconvdata/gconv-modules iconvdata
	
	echo "CFLAGS=\"-g $OPTIMIZE\" \\
	../configure  --prefix=/usr --enable-kernel=${MIN_KERNEL} --with-headers=/usr/src/linux-${KERNEL_HEADERS}/usr/include \\
	--enable-add-ons=libidn,nptl --disable-profile --infodir=/usr/share/info --mandir=/usr/share/man --with-tls \\
	--with-__thread --without-cvs --enable-bind-now $TARGET_OS" > ${CWD}/config-line.txt

	echo "Building GLIBC"
	CFLAGS="-g $OPTIMIZE" \
	../configure \
		--prefix=/usr \
		--libdir=/usr/lib${LIBDIRSUFFIX} \
		--enable-kernel=${MIN_KERNEL} \
		--with-headers=/usr/src/linux-${KERNEL_HEADERS}/usr/include \
		--enable-add-ons=libidn,nptl \
		--disable-profile \
		--infodir=/usr/share/info \
		--mandir=/usr/share/man \
		--with-tls \
		--with-__thread \
		--without-cvs \
		--enable-bind-now \
		$TARGET_OS
	
	make -j4 || exit 1
}

# install main content in PKG_TREE
install_pkg_tree() {
	if [ -d $PKG_TREE ] ; then
	echo "Removing stale package-tree content"
	rm -rf $PKG_TREE
	fi
	echo "Installing GLIBC in:"
	echo "$PKG_TREE"
	mkdir -p $PKG_TREE
	cd $TMP/glibc-$CVS_VERSION/build-glibc_$VERSION
	make install install_root=$PKG_TREE || exit 1
	make localedata/install-locales install_root=$PKG_TREE || exit 1
}

# strip ELF objects and static libs
strip_elf() {
	cd $PKG_TREE
	# Strip ELF objects. Don't use '--strip-unneeded' or you'll be sorry
	strip -g usr/lib${LIBDIRSUFFIX}/lib*.a
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip -g 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip -g 2> /dev/null
	# it's okay to strip binary executables hard
	( cd sbin ; strip * 2> /dev/null )
	( cd usr/bin ; strip * 2> /dev/null )
	( cd usr/sbin ; strip * 2> /dev/null )
	# and the gconv libs also
	( cd usr/lib${LIBDIRSUFFIX}/gconv ; strip * 2> /dev/null )
}

# install miscellaneous content
install_misc() {
	# Back to the sources dir to add some files/docs:
	cd $TMP/glibc-$CVS_VERSION
	# Install the config file for the Name Server Cache Daemon.
	mkdir -p $PKG_TREE/etc
	cat nscd/nscd.conf > $PKG_TREE/etc/nscd.conf.new
	## Install docs:
	( mkdir -p $PKG_TREE/usr/share/doc/glibc_$VERSION
		cp -a \
		BUGS CONFORMANCE COPYING COPYING.LIB FAQ INSTALL LICENSES NAMESPACE \
		NEWS NOTES PROJECTS README README.libm \
		$PKG_TREE/usr/share/doc/glibc_$VERSION
	)
	# copy this script into the docs, as well
	install -m0644 ${CWD}/${0##*/} $PKG_TREE/usr/share/doc/glibc_$VERSION
	# Fix info dir:
	rm $PKG_TREE/usr/share/info/dir
	${MAN_COMPRESS} $PKG_TREE/usr/share/info/*
	# This is junk
	rm $PKG_TREE/etc/ld.so.cache
	( cd $PKG_TREE
		find . -name "*.orig" -exec rm {} \;
	)
}
# Finished fixing up content in the main package tree
####

##############
# here's where the work starts
cd $TMP
create_dirs

# SKIP_REBUILD can be used to avoid a complete
# recompile when debugging or testing the build
############
if [[ -z ${SKIP_REBUILD} ]] ; then
	unpack_sources
	fix_source_perms
	# Apply patches; exit if any fail.
	apply_patches
	if [ ! $? = 0 ]; then
		exit 1
	fi
	build_sources
else
	echo "Skipping rebuild"
fi
############

# install and fixup the package-tree content
install_pkg_tree
strip_elf
install_misc

####################
# Now, create the packages

# Create glibc-zoneinfo package
( cd $PKG_TIME
	# Install timezone setup scripts
	mkdir -p var/lib/tpkg/setup
	cp -a $CWD/timezone/setup.timeconfig var/lib/tpkg/setup
	chown root:root var/lib/tpkg/setup/setup.timeconfig
	chmod 755 var/lib/tpkg/setup/setup.timeconfig
	mkdir -p usr/sbin
	cp -a $CWD/timezone/timeconfig usr/sbin
	chown root:root usr/sbin/timeconfig
	chmod 755 usr/sbin/timeconfig
	# move the timezone info
	mkdir -p usr/share
	( cd usr/share
	  mv $PKG_TREE/usr/share/zoneinfo .
	)
	# install the timezone tools
	mkdir -p usr/bin
	mv $PKG_TREE/usr/bin/tzselect usr/bin
	mkdir -p usr/sbin
	mv $PKG_TREE/usr/sbin/zdump $PKG_TREE/usr/sbin/zic usr/bin
	# install tzcode man-pages
	mkdir -p usr/share/man/man8
	( cd $TMP/glibc-$CVS_VERSION/timezone/tzcode
		cp tzselect.8 zdump.8 zic.8 $PKG_TIME/usr/share/man/man8
	)
	( cd usr/share/man
		find -type f -exec ${MAN_COMPRESS} {} \;
	)
	# installation items
	mkdir install
	cat $CWD/doinst.sh-glibc-zoneinfo > install/doinst.sh
	install_desc $CWD/pkg-desc/pkg-desc.glibc-zoneinfo
	# Create the package:
	rm -f $TMP/glibc-zoneinfo_$VERSION-noarch-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-zoneinfo_$VERSION-noarch-$BUILD.$PKG_FORMAT
)
# end creation of glibc-zoneinfo package

# Create the glibc-devel package
( cd $PKG_DEVEL
	# move the static libs
	mkdir -p usr/lib${LIBDIRSUFFIX}
	( cd usr/lib${LIBDIRSUFFIX}
		mv $PKG_TREE/usr/lib${LIBDIRSUFFIX}/*.a .
	)
	# and the headers
	mkdir -p usr/include
	( cd usr/include
		mv $PKG_TREE/usr/include/* .
	)
	rmdir $PKG_TREE/usr/include
	# info files
	mkdir -p usr/share/info
	( cd usr/share/info
		mv $PKG_TREE/usr/share/info/* .
	)
	rm -rf $PKG_TREE/usr/share/info
	# installation items
	mkdir install
	install_desc $CWD/pkg-desc/pkg-desc.glibc-devel
	# write the build info
	write_meta_data $PKG_DEVEL
	# Build the package:
	rm -f $TMP/glibc-devel_$VERSION-$ARCH-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-devel_$VERSION-$ARCH-$BUILD.$PKG_FORMAT
)
# end creation of glibc-devel package

# Create the main glibc-i18n package
# locale data is packaged in separate packages for each locale,
# but each requires the main glibc-i18n package to work
( cd $PKG_I18N
	# these are the bins needed for locales ~500K
	mkdir -p usr/bin
	(cd $PKG_TREE/usr/bin
		mv gencat iconv locale localedef $PKG_I18N/usr/bin
	)
	mkdir -p usr/sbin
	mv $PKG_TREE/usr/sbin/iconvconfig usr/sbin
	# gconv modules ~6.3MB
	mkdir -p usr/lib${LIBDIRSUFFIX}
	( cd usr/lib${LIBDIRSUFFIX}
		mv $PKG_TREE/usr/lib${LIBDIRSUFFIX}/gconv .
	)
	# move these assorted files
	mkdir -p $PKG_I18N/usr/share/i18n/locales
	# usr/share/i18n/locales ~2.4MB
	( cd $PKG_TREE/usr/share/i18n/locales
		mv POSIX iso* translit* i18n $PKG_I18N/usr/share/i18n/locales
	)
	mkdir -p $PKG_I18N/usr/share/locale
	mv $PKG_TREE/usr/share/locale/locale.alias $PKG_I18N/usr/share/locale
	# charmaps ~3.2MB
	mv $PKG_TREE/usr/share/i18n/charmaps $PKG_I18N/usr/share/i18n
	
	######## segregate locales into separate packages ########
	# This is still a somewhat crude method, but it makes it possible to install no locales 
	# or the users' choice of locales, without having to install *all* of the ~300MB i18n files.
	# Each locale package depends on the main i18n package, which contains all the tools 
	#needed. The main package also contains all the charmaps and gconv modules
	
	# start with a fresh listing of all the locales 
	: > $TMP/locales-list.tmp
	for path in usr/lib/locale usr/share/i18n/locales ; do
		( cd $PKG_TREE/$path
			for LOC in $(ls -1) ; do
				echo ${LOC} >> $TMP/locales-list.tmp
			done
		)
	done
	sort -u $TMP/locales-list.tmp |uniq -u > $TMP/locales-list
	rm -f $TMP/locales-list.tmp
	
	while read LOC ; do
		rm -rf $TMP/I18N/${LOC/_/-}
		mkdir -p $TMP/I18N/${LOC/_/-}
		if [[ $(find $PKG_TREE/usr/lib/locale -name "$LOC") != "" ]] ; then
			mkdir -p $TMP/I18N/${LOC/_/-}/usr/lib/locale
			find $PKG_TREE/usr/lib/locale -name "$LOC" -exec cp -a -L {} $TMP/I18N/${LOC/_/-}/usr/lib/locale \; 2> /dev/null
			rm -rf $PKG_TREE/usr/lib/locale/"$LOC"
		fi 
		if [[ $(find $PKG_TREE/usr/share/i18n/locales -name "$LOC") != "" ]] ; then
			mkdir -p $TMP/I18N/${LOC/_/-}/usr/share/i18n/locales
			find $PKG_TREE/usr/share/i18n/locales -name "$LOC" -exec cp -a -L {} $TMP/I18N/${LOC/_/-}/usr/share/i18n/locales \; 2> /dev/null
			rm -rf $PKG_TREE/usr/share/i18n/locales/"$LOC"
		fi
		# only package it if there are files
		#if [[ -d $TMP/I18N/$LOC ]] && [[ $(find $TMP/I18N/$LOC -type f) != "" ]] ; then
		if [[ -d $TMP/I18N/${LOC/_/-} ]] ; then
			echo -n "Processing locale: $LOC --> "
			# These alias dirs have a short name with no '_', '@' or '.'
			if [[ $(find $PKG_TREE/usr/share/locale -name "${LOC%_*}") != "" ]] ; then
				mkdir -p $TMP/I18N/${LOC/_/-}/usr/share/locale
				# these aren't removed right away, because each one may be included in more
				# than one package. The whole directory is removed when finished with this routine
				find $PKG_TREE/usr/share/locale -name "${LOC%_*}" -exec cp -a {} $TMP/I18N/${LOC/_/-}/usr/share/locale \; 2> /dev/null
			fi
			# start with fresh null settings
			lang= ; location= ; dialect= ; charset= ; DEPENDS=
			THIS_LOCALE=$(grep $LOC% $CWD/mapped-locales.txt)
			if [[ -z $THIS_LOCALE ]] ; then
				LOCA=${LOC%@*}
				THIS_LOCALE=$(grep $LOCA% $CWD/mapped-locales.txt)
			fi
			if [[ -z $THIS_LOCALE ]] ; then
				LOCA=${LOC%.*}
				THIS_LOCALE=$(grep $LOCA% $CWD/mapped-locales.txt)
			fi
			# extract the needed info from the retrieved line
			lang=$(echo $THIS_LOCALE |cut -f2 -d'%')
			location=$(echo $THIS_LOCALE |cut -f3 -d'%')
			dialect=$(echo $THIS_LOCALE |cut -f4 -d'%')
			charset=$(echo $THIS_LOCALE |cut -f5 -d'%')
			# localedef already copies needed dependencies
			#DEPENDS=$(echo $THIS_LOCALE |cut -f 6 -d%)
			
			( cd $TMP/I18N/${LOC/_/-}
				mkdir install
				# create a description file
				echo "glibc-locale-${LOC/_/-}: (GLIBC Language files for '$LOC' locale)" >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}:" >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}: This package contains the glibc language (i18n) files for the locale: '$LOC'" >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}: These files allow programs to use translated (internationalized) text." >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}: Location: $location" >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}: Language: $lang" >> install/pkg-desc
				if [[ -n $dialect ]] ; then
					echo "glibc-locale-${LOC/_/-}: Dialect: $dialect" >> install/pkg-desc
				else
					echo "glibc-locale-${LOC/_/-}:" >> install/pkg-desc
				fi
				if [[ -n $charset ]] ; then
					echo "glibc-locale-${LOC/_/-}: Charset: $charset" >> install/pkg-desc
				else
					echo "glibc-locale-${LOC/_/-}:" >> install/pkg-desc
				fi
				echo "glibc-locale-${LOC/_/-}:" >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}:" >> install/pkg-desc
				echo "glibc-locale-${LOC/_/-}: Packaged by $PACKAGER" >> install/pkg-desc
				# write a pkg-required file so that the package depedns on the main glibc-i18n package
				echo "# glibc-locale-${LOC/_/-}_$VERSION-$BUILD" >> install/pkg-required
				echo "# Requires:" >> install/pkg-required
				echo "glibc-i18n_$VERSION-$ARCH-$BUILD" >> install/pkg-required
				# add any dependencies on other locale packages ?
				# This doesn't seem to be necessary -the localedef commands which
				# create the locale copy any needed material from other locales
				#if [[ -n $DEPENDS ]] ; then
				#	(IFS=: ; for DEP in $DEPENDS ; do
				#		case $DEP in 
				#			i18n) true ;;
				#			iso14651_t1*) true ;;
				#			*)	echo glibc-locale-$DEP >> install/pkg-required ;;
				#		esac
				#	done
				#	)
				#fi
				
				# Use ARCH instead of 'noarch' as some files contain 'endian' binary data
				rm -f $TMP/I18N/glibc-locale-${LOC/_/-}_$VERSION-$ARCH-$BUILD.$PKG_FORMAT
				makepkg -l n -c n $TMP/I18N/glibc-locale-${LOC/_/-}_$VERSION-$ARCH-$BUILD.$PKG_FORMAT &> /dev/null
				echo glibc-locale-${LOC/_/-}_$VERSION-$ARCH-$BUILD.$PKG_FORMAT
			)
		else
			echo "Skipping locale: $LOC" >> $TMP/skipped-locales-list.txt
		fi
		rm -rf $TMP/I18N/${LOC/_/-}
	done < $TMP/locales-list
	
	rm -rf $PKG_TREE/usr/share/locale
	rm -rf $PKG_TREE/usr/lib/locale $PKG_TREE/usr/share/i18n
	######## finished segregating locale files
	# Now finish the main glibc-i18n package
	# installation items
	mkdir install
	install_desc $CWD/pkg-desc/pkg-desc.glibc-i18n.modular
	# Create the package:
	rm -f $TMP/glibc-i18n_$VERSION-noarch-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-i18n_$VERSION-noarch-$BUILD.$PKG_FORMAT
)
# end creation of glibc-i18n package

# Create the main glibc package from what is left
( cd $PKG_MAIN
	# Remove links from package tree -they are all handled by the doinst.sh scripts
	( cd $PKG_TREE
		find . -type l -exec rm {} \;
	)
	mv $PKG_TREE/* .
	mkdir -p etc/profile.d
	cp -a $CWD/profile.d/* etc/profile.d
	chown -R root:root etc
	chmod 755 etc/profile.d/*
	mkdir -p var/db/nscd var/run/nscd
	# put the libs in a bogus path for indirect handling
	# by the doinst.sh during installation
	( cd lib${LIBDIRSUFFIX}
		mkdir incoming
		mv *so* incoming
		mv incoming/libSegFault.so .
	)
	# installation items
	mkdir install
	install_desc $CWD/pkg-desc/pkg-desc.glibc
	install_doinst $CWD/doinst.sh-glibc
	# write the build info
	write_meta_data $PKG_MAIN
	# Build the package:
	rm -f $TMP/glibc_$VERSION-$ARCH-$BUILD.$PKG_FORMAT
	makepkg -l n -c n $TMP/glibc_$VERSION-$ARCH-$BUILD.$PKG_FORMAT
)
# end creation of glibc package

echo
echo "Finished building glibc packages in:"
echo "$TMP"
